import pandas
import os
import shutil
import datetime

# Calculate the value of the next raw value of the IBM Stock.
# The formula is:
#
# raw_prediction = moving_average_period * ma_prediction - sum_raw_over_period + first_raw_value
#
moving_average_period = 10

# Read prediction from disk
with open('.\\tmp\\prediction.txt', 'r') as f:
	line = f.read()
	ma_prediction = line.split('\t')[1]
	
# Read the last moving average value
latest_moving_average = pandas.read_csv('latest_moving_average.csv')

# Read the first in period and last raw values
latest_raw = pandas.read_csv('latest.csv')
first_raw_value = float(latest_raw[-1*(moving_average_period+1):-1*moving_average_period]['Close'])
last_raw_value = float(latest_raw[-1:]['Close'])

# Calculate the sum of the last period raw values
sum_raw_over_period = 0.0
for v in range(-1*(moving_average_period), 0):
	sum_raw_over_period += float(latest_raw[v-1:v]['Close'])
	
# Calculate the raw value
raw_prediction = float(moving_average_period) * float(ma_prediction) - sum_raw_over_period + first_raw_value
print 'The prediction for the IBM Stock for tomorrow is: ' + str(raw_prediction)

# Append today's actual and tomorrow's prediction to the history file
with open('history.csv', 'a') as f:
	f.write(str(last_raw_value) + '\n')
	f.write(str(datetime.date.today().strftime('%Y-%m-%d %X')))
	f.write(',' + str(raw_prediction) + ',')

# Calculate the absolute and relative errors of the previous prediction
history = pandas.read_csv('history.csv')
history['Actual'][-1:] = last_raw_value
actual = float(history['Actual'][-2:-1])
predicted = float(history['Prediction'][-2:-1])
abs_error = abs(actual - predicted)
rel_error = (abs(actual - predicted)*100.0/actual)
print 'The absolute error of yesterday\'s prediction was: %.4f'%abs_error
print 'The relative error of yesterday\'s prediction was: %.2f%%'%rel_error 


# Replace the template GeneXproTools file with the new file generated by
# this run because it contains a more up to date model.
os.remove('IBMStockClose.gep')
shutil.copy('./tmp/IBMStockClose_000001.gep', './IBMStockClose.gep')